#!/usr/bin/python

# Lost Cities
# Copyright 2003, David "Novalis" Turner <novalis@fsf.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import asynchat
import socket
import os
from time import sleep
from common.Connect import connect, disconnect

class ClientConnection (asynchat.async_chat):

    def __init__ (self, server, port, loginWindow):
        asynchat.async_chat.__init__ (self)
        self.loginWindow = loginWindow
        self.ibuffer = []
        self.set_terminator ("\n")
        self.create_socket (socket.AF_INET, socket.SOCK_STREAM)

        self.connect ((server, port))
        connect () #start the loop
                        
        self.mode = 'login'

    def handle_error (self):
        if self.mode == 'error':
            return
        self.loginWindow.storeEvent (self.loginWindow.loginError,
                                     None, "error in " + self.mode)
        self.mode = 'error'
        
    def handle_connect (self):
        self.loginWindow.storeEvent (self.loginWindow.connected, None)

    def challenge (self, username):
        self.sendl ("challenge", username)

    def sendl (self, *args):
        """Send space-separated items to the server."""
        data = ' '.join (map (str, args))
        self.push (data + "\n")
        
    def collect_incoming_data (self, data):
        """Buffer the data"""
        self.ibuffer.append (data)

    def setGame (self, game):
        self.game = game

    def setLobby (self, lobby):
        self.lobby = lobby
        self.mode = 'lobby'

    def found_terminator (self):
        """ This function calls the storeEvent function from FnQueue.  
        The named functions are be called from the main thread.  I
        can't just use the main QT mutex, because I will be calling
        arbitary functions, some of which may generate events. """
        
        response = ''.join (self.ibuffer)
        #print "got response: " + response
        self.ibuffer = []
        (command, arg) = response.split (' ', 1)

        if command == 'loginError':
            self.loginWindow.storeEvent (self.loginWindow.loginError,
                                         None, arg)
            return

        if command == "loggedIn":
            self.mode = 'lobby'
            self.loginWindow.storeEvent (self.loginWindow.initLobby,
                                         self.setLobby, self)
            sleep (1)
            return
        
        if command == 'challenge':
            self.lobby.storeEvent (self.lobby.gotChallenge, None, arg)
            return
             
        if command == "user":
            self.lobby.storeEvent (self.lobby.user, None, arg.split ('\0'))
            return

        if command == "chat":
            self.lobby.storeEvent (self.lobby.gotChat, None, *(arg.split ('\0')))
        
        if command == "playing":
            self.mode = 'playing'
            self.lobby.storeEvent (self.lobby.initGame, self.setGame, arg)
            sleep (5) #this delays loading the cards so that two people
            #can play on one machine
            return
        
        if command == 'gameOver':
            self.game.storeEvent (self.game.gameOver, None)
            return

        if command == "turn":
            self.game.storeEvent (self.game.turn, None)
            return

        if command == "played":
            self.game.storeEvent (self.game.theyPlayed, None, arg)
            return

        if command == "discarded":
            self.game.storeEvent (self.game.theyDiscarded, None, arg)
            return
            
        if command == "notturn":
            self.game.storeEvent (self.game.turnOver, None)
            return

        if command == 'youdrew':
            self.game.storeEvent (self.game.drew, None, arg)
            return

        if command == 'theydrew':
            self.game.storeEvent (self.game.theyDrew, None, arg)
            return
