#!/usr/bin/python

# Lost Cities
# Copyright 2003, David "Novalis" Turner <novalis@fsf.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

from qt import *

import os
from time import sleep
from ConfigParser import ConfigParser

from QUtility import *
from common.Connect import disconnect

from Login import Login
from LobbyImpl import LobbyImpl
from ClientConnection import ClientConnection
from FnQueue import FnQueue

class LoginImpl (Login, FnQueue):
    def __init__ (self, app):
        Login.__init__ (self)

        self.app = app
        #servers are of the form server:port
        self.servers.setValidator (QRegExpValidator (QRegExp ("[\\w-]([\\w.-]*):[0-9]+"), None))

        config = self.loadConfigFile ()
        #load the stored servers from the config file
        if config.has_section ("servers"):
            i = 0
            while 1:
                try:
                    server = config.get ("servers", "server%d" % i)
                    self.servers.addItem (server)
                except:
                    break
                i += 1

        #load the stored username and password if any
        if config.has_section ("login"):
            try:
                self.usernameInput.setText (config.get ("login", "username"))
            except:
                pass
            try:
                self.passwordInput.setText (config.get ("login", "password"))
            except:
                pass
                
            
        self.pendingEvents = []
        self.startTimer (5)

    def initLobby (self, conn):
        #save server list
        config = self.loadConfigFile ()
        if not config.has_section ("servers"):
            config.add_section ("servers")

        for i in range (0, self.servers.count ()):
            config.set ("servers", "server%d" % i, self.servers.text (i))

        self.saveConfigFile (config)

        self.connectDialog.accept ()
        self.hide ()
        l = LobbyImpl (conn, self)
        l.show ()
        return l

    def needData (self, data):
        warning (self, "You need to enter a " + data + ".")


    def getUsername (self):
        """Store the username and password to send when we are connected"""
        self.username = self.usernameInput.text ()
        if not self.username:
            self.needData ("username")
            return
        self.password = self.passwordInput.text ()
        if not self.password:
            self.needData ("password")
            return

    def configFileName (self):
        """Return the name of the configuration file"""
        return os.path.expanduser ("~/.lostCities")
    
    def loadConfigFile (self):
        config = ConfigParser ()
        config.read (self.configFileName ())
        return config

    def saveConfigFile (self, config):
        f = open (self.configFileName (), "w")
        config.write (f)
        f.close ()
        
    def saveUsername (self):
        """save the username and password, if needed"""
        config = self.loadConfigFile ()
        if not config.has_section ("login"):
            config.add_section ("login")
        if self.saveUsernameCheck.isChecked ():
            config.set ("login", "username", self.username)
        if self.savePasswordCheck.isChecked ():
            config.set ("login", "password", self.password)

        self.saveConfigFile (config)

    def login (self):
        """This function is called when the "Login" button is checked"""
        
        #we know that server is of the correct form because of the
        #validator
        server = str (self.servers.currentText ())
        (addr, port) = server.split (":", 1)
        port = int (port)

        self.getUsername ()
        self.saveUsername ()

        #tell the user that we are connecting
        self.connectDialog = ConnectDialog (self)
        self.connectDialog.show ()
        self.storeEvent (self.connectServer, None, addr, port)

    def connectServer (self, addr, port):
        self.conn = ClientConnection (addr, port, self)

    def cancelConnect (self):
        self.conn.close ()
        
    def connected (self):
        if self.createAccountCheck.isChecked ():
            self.conn.sendl ("createAccount", self.username)
        else:
            self.conn.sendl ("login", self.username)
        self.conn.sendl ("password", self.password)

    def loginError (self, err):
        self.connectDialog.reject ()
        #self.cancelConnect ()
        warning (self, "Login error: " + err)

    def focusPassword (self):
        self.passwordInput.setFocus ()

    def closeEvent (self, e):
        print "closed login"
        self.app.quit ()
        disconnect ()
        Login.closeEvent (self, e)
        
class ConnectDialog (QDialog):
    def __init__ (self, parent):
        QDialog.__init__ (self, parent, "Connecting ...")

        layout = QVBoxLayout (self, 11, 6)
        
        self.label = QLabel (self)
        self.label.setText ("Connecting...")
        layout.addWidget (self.label)
        
        self.cancel = QPushButton (self)
        self.cancel.setText ("Cancel")
        layout.addWidget (self.cancel)

        self.resize (QSize (195,102).expandedTo (self.minimumSizeHint()))

        self.connect (self.cancel, SIGNAL ("clicked()"), SLOT ("reject()"))
        
    def reject (self):
        self.parent ().cancelConnect ()
        return QDialog.reject (self)

    def accept (self):
        return QDialog.accept (self)
        
