#!/usr/bin/python

# Lost Cities
# Copyright 2003, 2004, David "Novalis" Turner <novalis@fsf.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

from qt import *

from LostCities import LostCities
from ClientConnection import ClientConnection
from FnQueue import FnQueue

from common.Connect import connect, disconnect
from common.Utility import *
from QUtility import *

class LostCitiesImpl (LostCities, FnQueue):
    def __init__ (self, conn, opponent): 
        LostCities.__init__ (self)

        self.conn = conn
        self.hand = []
        self.initLists ()

        self.playedCards = [[],[],[],[],[]]
        self.discardCards = [[],[],[],[],[]]
        self.opponentPlayedCards = [[],[],[],[],[]]

        self.inDeck = 44
        self.opponentName = opponent
        
        self.opponentNamePosessive = opponent + "'s"
        self.theyPlayedLabelLabel.setText (self.opponentName + " played:")
        self.theyDrewLabelLabel.setText (self.opponentName + " drew:")

        self.pendingEvents = []
        self.startTimer (1)


    def sendl (self, *args):
        self.conn.sendl (*args)

    def initLists (self):
        self.grey = self.paletteBackgroundColor ()
        self.discardLists = [self.greenDiscarded, self.redDiscarded,
                             self.yellowDiscarded, self.purpleDiscarded,
                             self.blueDiscarded]

        self.playedLists = [self.greenPlayed, self.redPlayed,
                             self.yellowPlayed, self.purplePlayed,
                             self.bluePlayed]

        self.opponentPlayedLists = [self.greenOpponent, self.redOpponent,
                              self.yellowOpponent, self.purpleOpponent,
                              self.blueOpponent]

        self.scores = [self.greenScore, self.redScore,
                       self.yellowScore, self.purpleScore,
                       self.blueScore]

        self.opponentScores = [self.greenOpponentScore, self.redOpponentScore,
                               self.yellowOpponentScore,
                               self.purpleOpponentScore, self.blueOpponentScore]

        self.drawButtons = [self.greenDraw, self.redDraw, self.yellowDraw,
                            self.purpleDraw, self.blueDraw]

        self.colors = [QColor (165, 255, 165), #green
                       QColor (255, 179, 179), #red
                       QColor (255, 255, 164), #yellow
                       QColor (230, 180, 255), #purple
                       QColor (185, 192, 255)] #blue

        self.hcolors = [QColor (0, 192, 0), #green
                       QColor (224, 0, 0), #red
                       QColor (192, 192, 0), #yellow
                       QColor (192, 0, 192), #purple
                       QColor (0, 0, 255)] #blue

        self.colorNames = ['green', 'red', 'yellow', 'purple', 'blue']
        
    def drew (self, card):
        (suit, rank) = parseCard (card)

        rank = self.colorNames [suit] + ' ' + rank
        item = HandItem (self.handList, self.colors [suit], self.hcolors [suit], rank, card)
        self.handList.insertItem (item)
        if self.handList.selectedItem () is None:
            self.handList.setSelected (item, 1)

        self.turnOver ()
            
    def theyPlayed (self, card):

        (suit, rank) = parseCard (card)

        PlayedItem (self.opponentPlayedLists [suit], rank)

        self.theyPlayedLabel.setText (self.colorNames [suit] + " " + rank)
        self.theyPlayedLabel.setPaletteBackgroundColor (self.colors [suit])
                
        self.calcScore (self.opponentPlayedCards [suit], self.opponentScores [suit], card)
        total = 0
        for x in self.opponentScores:
            total += int (str (x.text ()))
        self.opponentTotalScore.setText (str (total))

    def calcScore (self, played, score, card):
        played.append (card)
        mul = 1
        sum = 0
        
        for x in played:
            (suit, rank) = parseCard (x)
            if rank == "handshake":
                mul += 1
            else:
                sum += int (rank)

        #bonus for eight or more cards:
        if len (played) >= 8:
            bonus = 20
        else:
            bonus = 0
        total = (sum - 20) * mul + bonus
        score.setText (str (total))

    def theyDiscarded (self, card):

        (suit, rank) = parseCard (card)

        PlayedItem (self.discardLists [suit], rank)
        
        self.discardCards [suit].append (int (card))
        self.theyPlayedLabel.setText (self.colorNames [suit] + " " + rank)
        self.theyPlayedLabel.setPaletteBackgroundColor (self.colors [suit])
        self.theyPlayedLabelLabel.setText (self.opponentName + " discarded:")
        
    def theyDrew (self, suit):
        if suit == "deck":
            self.inDeck -= 1
            self.drawFromDeck.setText ("Draw From Deck (%d cards left)" % self.inDeck)
            self.theyDrewLabel.setText ("from the deck")
            self.theyDrewLabel.setPaletteBackgroundColor (self.grey)

        else:
            suit = int (suit)
            #assume server won't screw up
            card = self.discardCards [suit].pop ()
            self.discardLists [suit].takeItem (self.discardLists [suit].lastItem ())
            (suit, rank) = parseCard (card)
            self.theyDrewLabel.setText (self.colorNames [suit] + " " + rank)
            self.theyDrewLabel.setPaletteBackgroundColor (self.colors [suit])
        self.turn ()
    
    def turnOver (self):
        for b in self.drawButtons:
            b.setEnabled (0)
        self.drawFromDeck.setEnabled (0)
        self.discardButton.setEnabled (0)
        self.playButton.setEnabled (0)
        self.phase.setText (self.opponentNamePosessive + " turn")
        self.theyDrewLabel.setText ("")
        self.theyPlayedLabel.setText ("")
        self.theyDrewLabel.setPaletteBackgroundColor (self.grey)
        self.theyPlayedLabel.setPaletteBackgroundColor (self.grey)
        
    def turn (self):
        for c in self.drawButtons:
            c.setEnabled (0)
        self.drawFromDeck.setEnabled (0)
        self.playButton.setEnabled (1)
        self.discardButton.setEnabled (1)
        self.phase.setText ("Your turn, play")
        
    def drawPhase (self):
        """ After the player has played, set up the interface
        for them to draw"""
        for c in self.drawButtons:
            c.setEnabled (1)
        self.drawFromDeck.setEnabled (1)
        self.discardButton.setEnabled (0)
        self.playButton.setEnabled (0)
        self.phase.setText ("Your turn, draw")

    def closeEvent (self, e):
        self.conn.close ()
        disconnect ()
        self.conn = None
        return LostCities.closeEvent (self, e)

    def discard_from_hand (self):
        cardItem = self.handList.selectedItem ()
        if not cardItem:
            return
        card = cardItem.card
        
        self.sendl ("discard", card)

        (suit, rank) = parseCard (card)
        
        #update our state and interface
        self.discardCards [suit].append (card)

        PlayedItem (self.discardLists [suit], rank)
       
        self.handList.takeItem (cardItem)
        self.drawPhase ()

        
    def play_from_hand (self):
        """Play the selected card from your hand."""

        cardItem = self.handList.selectedItem ()
        if not cardItem:
            return
        card = cardItem.card

        #is this a legal play?
        suit = card / 13
        rank = card % 13
        if not canPlayCard (suit, rank, self.playedCards):
            warning (self,  "You can't play that card")
            return

        self.sendl ("play", card)

        #update our state and interface
        if rank < 3:
            rank = "handshake"
        else:
            rank = str (rank - 2)

        PlayedItem (self.playedLists [suit], rank)
        
        self.handList.takeItem (cardItem)

        self.calcScore (self.playedCards [suit], self.scores [suit], card)
        total = 0
        for x in self.scores:
            total += int (str (x.text ()))
        self.totalScore.setText (str (total))
        self.drawPhase ()

    def draw_from_deck (self):
        """The slot for the drawFromDeck button"""
        self.inDeck -= 1
        self.drawFromDeck.setText ("Draw From Deck (%d cards left)" % self.inDeck)

        self.sendl ("drawDeck dummyArg")

    def gameOver (self):
        
        for c in self.drawButtons:
            c.setEnabled (0)
        self.drawFromDeck.setEnabled (0)
        self.discardButton.setEnabled (0)
        self.playButton.setEnabled (0)
        
        warning (self, "Game over.  Final scores:\nYou:  " +
                 str (self.totalScore.text ()) + "\n" + self.opponentName +
                 ":  " + str (self.opponentTotalScore.text ()))

        #go back to lobby
        self.conn.lobby.show ()
        self.conn.mode = 'lobby'
        self.close ()

    def draw0 (self):
        self.drawDiscard (0)
    def draw1 (self):
        self.drawDiscard (1)
    def draw2 (self):
        self.drawDiscard (2)
    def draw3 (self):
        self.drawDiscard (3)
    def draw4 (self):
        self.drawDiscard (4)

    def drawDiscard (self, pile):
        """ User clicked one of the five draw buttons below the discard piles"""
        if len (self.discardCards [pile]) == 0:
            return

        card = self.discardCards [pile].pop ()
        self.discardLists [pile].takeItem (self.discardLists [pile].lastItem ())
        self.sendl ("drawDiscard", pile)
        self.drew (card)


#A global counter for sort keys
k = 0
handShakePixmap = None

#fixme: track card internal to here
class PlayedItem (QListViewItem):
    def __init__ (self, parent, text):
        global k
        k = k + 1
        self.sortKey = QString ("%05d" % k)
        QListViewItem.__init__ (self, parent)

#used to use graphics here, but didn't have them in handitem, so...
##         if text == "handshake":
##             global handShakePixmap
##             if not handShakePixmap:
##                 handShakePixmap = QPixmap ("handshake.png")
                
##             self.setPixmap (0, handShakePixmap)
##        else:
        
        self.setText (0, text)
        parent.sort ()
        
    def key (self, col, asc):
        """Played and discarded cards are sorted by play/discard order.
        We use a global counter to determine order. """
        return self.sortKey


#fixme?  Use handshake graphics in HandItem
class HandItem (QListViewItem):
    def __init__ (self, parent, color, hcolor, text, card):
        QListViewItem.__init__ (self, parent)
        self.color = color
        self.hcolor = hcolor

        self.setText (0, text)

        self.card = int (card)
        self.sortKey = QString ("%02d" % self.card)
        

    def paintCell (self, p, cg, column, width, alignment):
        _cg = QColorGroup (cg)
        base = QColor (_cg.base ())
        highlight = QColor (_cg.highlight ())

        _cg.setColor (QColorGroup.Base, self.color)
        _cg.setColor (QColorGroup.Highlight, self.hcolor)

        QListViewItem.paintCell (self, p, _cg, column, width, alignment )

        _cg.setColor( QColorGroup.Base, base)
        _cg.setColor( QColorGroup.Base, highlight)

    def key (self, col, asc):
        return self.sortKey
