#!/usr/bin/python

# Lost Cities
# Copyright 2003, David "Novalis" Turner <novalis@fsf.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import asynchat

class Connection (asynchat.async_chat):

    def __init__ (self, (conn, addr), owner):
        asynchat.async_chat.__init__ (self, conn=conn)
        self.ibuffer = []
        self.set_terminator ("\n")
        self.conn = conn
        self.owner = owner
        self.mode = 'login'
        
    def sendl (self, *args):
        data = ' '.join (map (str, args))
        self.push (data + "\n")
        
    def handle_close (self):
        self.owner.closeUserConn (self.username)
        asynchat.async_chat.handle_close (self)
        
    def collect_incoming_data (self, data):
        """Buffer the data"""
        self.ibuffer.append (data)

    def playing (self, user):
        self.sendl ("playing", user)
        self.mode = 'playing'

    def setPlayer (self, player):
        self.player = player

    def found_terminator (self):
        response = ''.join (self.ibuffer)
        #print "got response: " + response
        self.ibuffer = []

        (command, arg) = response.split (' ', 1)

        if self.mode == 'login' and command == 'createAccount':
            self.username = arg
            self.loginMode = self.owner.createAccount
            return            
        
        if self.mode == 'login' and command == 'login':
            self.username = arg
            self.loginMode = self.owner.login
            return
        
        if self.mode == 'login' and command == 'password':
            self.password = arg
            if not self.loginMode (self.username, self.password, self):
                self.sendl ("loginError", "Login failed")
                self.close ()
            else:
                self.mode = 'lobby'                
            return
        
        if self.mode == 'lobby' and command == 'chat':
            self.owner.chat (self.username, arg)
            return
            
        if self.mode == 'lobby' and command == 'challenge':
            if not self.owner.challenge (self.username, arg):
                #this should happen only when the user logs off, so the client
                #will probably ignore it
                self.sendl ("connectionError", "self.opponent does not exist")
            return
        
        if command == 'play':
            self.player.playCard (int (arg))
            return
        
        if command == 'discard':
            self.player.discard (int (arg))
            return
        
        if command == 'drawDeck':
            self.player.drawDeck ()
            return
        
        if command == 'drawDiscard':
            self.player.drawDiscard (int (arg))
            return

        print "Unhandled command!"
